<?php
/**
* This file is part of the Agora-Project Software package.
*
* @copyright (c) Agora-Project Limited <https://www.agora-project.net>
* @license GNU General Public License, version 2 (GPL-2.0)
*/


/*
 * Controleur principal
 */
abstract class Ctrl
{
	//Propriétés de base
	const moduleName=null;
	public static $moduleOptions=array();
	public static $agora, $curUser, $curSpace;
	public static $isMainPage=false;			//Page principale avec barre de menu, messenger, etc (false pour les iframe)
	public static $userHasConnected=false;		//idem : l'user vient de s'identifier / connecter
	public static $curContainer=null;			//idem : objet conteneur courant (dossier, sujet, etc)
	public static $curTimezone=null;			//Timezone courante
	protected static $initCtrlFull=true;		//Initialisation complete du controleur (connexion d'user, selection d'espace, etc)
	protected static $folderObjectType=null;	//Module avec une arborescence
	protected static $cachedObjects=array();	//Objets mis en cache !
	//Messages de Notifications & Triggers Javascript (cf. Vues)
	private static $_msgNotif=array();
	private static $_jsTriggers=array();

	/*
	 * Initialise le controleur principal : session, parametrages, connexion de l'user, selection de l'espace, etc
	 */
	public static function initCtrl()
	{
		////	Init la session
		if(defined("db_name"))	{session_name("Agora_".db_name);}//si plusieurs espaces sur le même serveur
		session_cache_limiter("nocache");
		session_start();

		////	Déconnexion : reinit les valeurs de session
		if(Req::isParam("disconnect")){
			$_SESSION=[];
			session_destroy();
			setcookie("AGORAP_PASS", null, (time()-31536000), null, null, false, true);//reinit
		}

		////	Controle la mise à jour PUIS Récup le parametrage de l'espace (apres "session_start")
		DbUpdate::lauchUpdate();
		self::$agora=self::getObj("agora");

		////	Init le fuseau horaire
		self::$curTimezone=array_search(self::$agora->timezone,Tool::$tabTimezones);
		if(empty(self::$curTimezone))	{self::$curTimezone="Europe/Paris";}
		date_default_timezone_set(self::$curTimezone);

		////	Init l'user et l'espace courant (après init de session). Recup au besoin l'id du dossier courant
		$userId=(!empty($_SESSION["_idUser"])) ? $_SESSION["_idUser"] : "isGuest";
		$spaceId=(!empty($_SESSION["_idSpace"])) ? $_SESSION["_idSpace"] : null;
		self::$curUser=self::getObj("user",$userId);
		self::$curSpace=self::getObj("space",$spaceId);

		////	Init complète du controleur : connexion de l'user/invité, selection d'espace, etc
		if(static::$initCtrlFull==true)
		{
			////	Connection d'un user? Selection d'un espace? Init les stats?
			self::userConnection();
			if(self::isHost())  {Host::connectStats();}//Toujours après "userConnection()" !
			self::curSpaceSelection();
			////	Chargement des trads et des "locales"
			Txt::loadTrads();
			////	Affichage administrateur demandé (switch) ?
			if(self::$curUser->isAdminCurSpace() && Req::isParam("displayAdmin"))	{$_SESSION["displayAdmin"]=(bool)Req::getParam("displayAdmin");}
			////	Affichage des utilisateurs : space/all
			if(empty($_SESSION["displayUsers"]))	{$_SESSION["displayUsers"]="space";}
			////	Objet à charger et à controler (tjs après chargement des trads!)
			if(strlen(Req::getParam("targetObjId"))>0)	{$targetObj=self::getTargetObj();}//Dossier/conteneur courant ou autre objet
			elseif(static::$folderObjectType!==null)	{$targetObj=self::getTargetObj(static::$folderObjectType."-1");}//Dossier racine par défaut
			////	Charge le dossier/conteneur courant & controle d'accès
			if(isset($targetObj) && is_object($targetObj) && !empty($targetObj->_id)){
				if($targetObj::isContainer())  {self::$curContainer=$targetObj;}
				if($targetObj->readRight()==false){
					if(static::$isMainPage==true)	{self::redir("?ctrl=".Req::$curCtrl);}//redirige vers controleur principal
					else							{self::noAccessExit();}//message d'erreur
				}
			}
		}
	}

	/*
	 * Recupère un objet, déjà en cache?
	 */
	public static function getObj($objectType, $objIdOrValues=null, $updateCachedObj=false)
	{
		//Objet inexistant / existant (recup/update en cache?)
		$MdlObject="Mdl".ucfirst($objectType);
		if(empty($objIdOrValues))	{return new $MdlObject();}
		else
		{
			$objId=(!empty($objIdOrValues["_id"]))  ?  $objIdOrValues["_id"]  :  (int)$objIdOrValues;
			$objectCachedKey=$MdlObject::objectType."-".$objId;
			//Ajoute/Update l'objet en cache?
			if(!isset(self::$cachedObjects[$objectCachedKey]) || $updateCachedObj==true)	
				{self::$cachedObjects[$objectCachedKey]=new $MdlObject($objIdOrValues);}
			//Retourne l'objet en cache
			return self::$cachedObjects[$objectCachedKey];
		}
	}

	/*
     * Génère une vue à partir d'un fichier et des parametres $datas
     */
	public static function getVue($filePath, $datas=array())
	{
		if(file_exists($filePath)){
			ob_start();				//Démarre la temporisation de sortie
			extract($datas);		//On rend les $datas accessibles à la vue ($data["monParametre"] devient $monParametre)
			require $filePath;		//Inclut le fichier vue
			return ob_get_clean();	//Renvoie du tampon de sortie
		}
		else{throw new Exception("File '".$filePath."' unreachable");}
	}

	/*
	 * Affiche une page complete (ensemble de vues)
	 */
	protected static function displayPage($fileMainVue=null, $vDatasMainVue=array())
	{
		//Init
		$vDatas["mainContent"]=$vDatas["headerMenu"]=$vDatas["messengerLivecounter"]=null;
		////	CORPS DE LA PAGE (sauf si validation de formulaire : "lightboxClose")
		if($fileMainVue!=null){
			$pathVue=(strstr($fileMainVue,Req::commonPath)==false) ? Req::getCurModPath() : null;//"app/Common/" déjà précisé?
			$vDatas["mainContent"]=self::getVue($pathVue.$fileMainVue, $vDatasMainVue);
		}
		////	Page principale
		if(static::$isMainPage==true)
		{
			//Wallpaper & Logo footer
			if(!empty(self::$curSpace->wallpaper))	{$vDatas["pathWallpaper"]=CtrlMisc::pathWallpaper(self::$curSpace->wallpaper);}
			elseif(!empty(self::$agora->wallpaper))	{$vDatas["pathWallpaper"]=CtrlMisc::pathWallpaper(self::$agora->wallpaper);}
			else									{$vDatas["pathWallpaper"]=CtrlMisc::pathWallpaper();}
			$vDatas["pathLogoUrl"]=(empty(self::$agora->logoUrl))  ?  OMNISPACE_URL_PUBLIC  :  self::$agora->logoUrl;
			$vDatas["pathLogoTitle"]=OMNISPACE_URL_LABEL."<br>".Txt::trad("FOOTER_pageGenerated")." ".round((microtime(true)-TPS_EXEC_BEGIN),3)." sec.";
			//HeaderMenu & MessengerLivecounter
			if(static::moduleName!="offline")
			{
				//Mise à jour récente : notification dans le "pageFooterHtml" pour l'admin de l'espace
				if(self::$curUser->isAdminCurSpace() && self::$curUser->previousConnection<strtotime(self::$agora->dateUpdateDb))
					{self::$agora->footerHtml="<span id='footerHtmlUpdate'>".Txt::trad("NOTIF_update")." ".Txt::displayDate(self::$agora->dateUpdateDb,"dateMini")." : v".VERSION_AGORA."</span><script>$('#footerHtmlUpdate').effect('pulsate',{times:5},10000);</script>";}
				//Espace Disk
				$vDatasHeader["diskSpacePercent"]=ceil((File::datasFolderSize()/limite_espace_disque)*100);
				$vDatasHeader["diskSpaceAlert"]=($vDatasHeader["diskSpacePercent"]>70) ? true : false;
				//Plugin "shortcuts" de chaque module
				$vDatasHeader["pluginsShortcut"]=array();
				$pluginParams=array("type"=>"shortcut");
				foreach(self::$curSpace->moduleList() as $tmpModule){
					if(method_exists($tmpModule["ctrl"],"plugin"))	{$vDatasHeader["pluginsShortcut"]=array_merge($vDatasHeader["pluginsShortcut"],$tmpModule["ctrl"]::plugin($pluginParams));}
				}
				//Validation d'inscription d'utilisateurs
				$vDatasHeader["userInscriptionValidation"]=(self::$curUser->isAdminCurSpace() && Db::getVal("SELECT count(*) FROM ap_userInscription WHERE _idSpace=".(int)self::$curSpace->_id)>0);
				//Liste des modules (Url, Description, Libellé, Class de l'icone)
				$vDatasHeader["moduleList"]=self::$curSpace->moduleList();
				foreach($vDatasHeader["moduleList"] as $moduleKey=>$tmpModule)	{$vDatasHeader["moduleList"][$moduleKey]["isCurModule"]=($tmpModule["moduleName"]==static::moduleName)  ?  true  : false;}
				//retourne les vues du Header & du livecounter
				$vDatas["headerMenu"]=self::getVue(Req::commonPath."Vue/VueHeaderMenu.php",$vDatasHeader);
				if(self::$curUser->messengerEnabled())	{$vDatas["messengerLivecounter"]=self::getVue(Req::commonPath."Vue/VueMessengerLivecounter.php");}
			}
		}
		////	Notifications passées en Get/Post ?
		if(Req::isParam("msgNotif")){
			foreach(Req::getParam("msgNotif") as $tmpNotif){
				if(Txt::isTrad($tmpNotif))	{self::addNotif(Txt::trad($tmpNotif));}//Texte en trad
				else						{self::addNotif($tmpNotif);}//Texte brute
			}
		}
		////	Affiche le résultat
		$vDatas["msgNotif"]=self::$_msgNotif;
		$vDatas["jsTriggers"]=self::$_jsTriggers;
		$vDatas["skinCss"]=(!empty(self::$agora->skin) && self::$agora->skin=="black")  ?  "black"  :  "white";
		
		echo self::getVue(Req::commonPath."Vue/VueStructure.php",$vDatas);
	}

	/*
	 * Ajoute lance un trigger JavaScript
	 */
	public static function addJsTrigger($jsTrigger)
	{
		self::$_jsTriggers[]=$jsTrigger;
	}

	/*
	 * Ajoute une notification à afficher
	 * type : "info" / "success" / "warning"
	 */
	public static function addNotif($message, $type="notice")
	{
		//Traduction à récupérer ?
		if(Txt::isTrad($message))	{$message=Txt::trad($message);}
		//Si le message n'est pas déjà affiché : on l'ajoute
		if(Tool::arraySearch(self::$_msgNotif,$message)==false){
			self::$_msgNotif[]=array("message"=>$message,"type"=>$type);
		}
	}

	/*
	 * Redirige une page (avec si besoin des notifications)
	 */
	public static function redir($url)
	{
		header("Location: ".$url.self::urlMsgNotif());
		exit;
	}

	/*
	 * Recupère les "msgNotif" pour les ajouter à l'url
	 */
	public static function urlMsgNotif()
	{
		$urlMsgNotif=null;
		if(!empty(self::$_msgNotif)){
			foreach(self::$_msgNotif as $message)	{$urlMsgNotif.="&msgNotif[]=".urlencode($message["message"]);}
		}
		return $urlMsgNotif;
	}

	/*
	 * Ferme le lightbox (exple: apres enregistrement de formulaire)
	 */
	public static function lightboxClose($urlMoreParms=null)
	{
		//Prépare le trigger JS de fermeture de lightbox : avec reload la page principale, et si besoin des notifications et parametres spécifiques
		self::addJsTrigger("lightboxClose(null, \"".self::urlMsgNotif().$urlMoreParms."\");");
		//Affiche uniquement la structure de page avec les triggers JS (donc pas de vue principale)
		static::displayPage();
		exit;
	}

	/*
	 * Affiche "element inaccessible" (ou autre) & sortie de script
	 */
	public static function noAccessExit($message=null)
	{
		if($message===null)  {$message=Txt::trad("inaccessibleElem");}
		echo "<h2><img src='app/img/important.png' style='vertical-align:middle;'> ".$message."</h2>";
		exit;
	}

	/*
	 * Vérif si on est sur un Host
	 */
	public static function isHost()
	{
		return defined("HOST_DOMAINE");
	}


	/***************************************************************************************************************************/
	/*******************************************	SPECIFIC METHODS	********************************************************/
	/***************************************************************************************************************************/

	/*
	 * Valide au besoin la connexion d'un utilisateur
	 */
	public static function userConnection()
	{
		$connectViaForm=(Req::isParam(["connectLogin","connectPassword"])) ? true : false;
		$connectViaCookie=(!empty($_COOKIE["AGORAP_LOG"]) && !empty($_COOKIE["AGORAP_PASS"]) && Req::isParam("disconnect")==false) ? true : false;
		if(self::$curUser->isUser()==false && ($connectViaForm==true || $connectViaCookie==true))
		{
			////	CONTROLES DE CONNEXION
			//Connexion demandé ou auto ?
			if($connectViaForm==true)		{$login=Req::getParam("connectLogin");  $passwordSha1=MdlUser::passwordSha1(Req::getParam("connectPassword"));}
			elseif($connectViaCookie==true)	{$login=$_COOKIE["AGORAP_LOG"];			$passwordSha1=$_COOKIE["AGORAP_PASS"];}
			//Identification + recup des infos sur l'user
			$sqlPasswordSha1="AND password=".Db::format($passwordSha1);
			if(self::isHost())  {$sqlPasswordSha1=Host::sqlPassword(Req::getParam("connectPassword"),$sqlPasswordSha1);}//SHORT.C?
			$tmpUser=Db::getLine("SELECT * FROM ap_user WHERE login=".Db::format($login)." ".$sqlPasswordSha1);
			//Aucun user correspondant : tente une identification LDAP et creation d'user à la volee
			if(empty($tmpUser) && $connectViaForm==true)  {$tmpUser=MdlUser::ldapConnectCreateUser(Req::getParam("connectLogin"),Req::getParam("connectPassword"));}
			//...Toujours pas connecté : message d'erreur et déconnexion
			if(empty($tmpUser))   {self::addNotif("NOTIF_identification");  self::redir("?disconnect=1");}
			//Compte en cours d'utilisation sur un autre poste avec une autre ip (pas de controle sur l'appli)
			if(Tool::isMobileApp()==false){
				$autreIpConnected=Db::getVal("SELECT count(*) FROM ap_userLivecouter WHERE _idUser=".(int)$tmpUser["_id"]." AND date > '".(time()-60)."' AND ipAdress NOT LIKE '".$_SERVER["REMOTE_ADDR"]."'");
				if($autreIpConnected>0)   {self::addNotif("NOTIF_presentIp");  self::redir("?disconnect=1");}
			}
			////	VALIDATION DE L'UTILISATEUR
			//Connexion validé :reinitialise la session, charge l'utilisateur courant, etc
			$_SESSION=array();
			$_SESSION["_idUser"]=(int)$tmpUser["_id"];
			$previousConnection=(!empty($tmpUser["lastConnection"]))  ?  $tmpUser["lastConnection"]  :  time();
			Db::query("UPDATE ap_user SET lastConnection='".time()."', previousConnection=".Db::format($previousConnection)." WHERE _id=".(int)$tmpUser["_id"]);
			self::$curUser=self::getObj("user",$_SESSION["_idUser"]);
			self::$userHasConnected=true;
			self::addLog("connexion");
			//Récupère les préférences
			foreach(Db::getTab("select * from ap_userPreference where _idUser=".self::$curUser->_id) as $tmpPref)
				{$_SESSION["pref"][$tmpPref["keyVal"]]=$tmpPref["value"];}
			//Enregistre login & password pour une connexion auto(10A)
			if(Req::isParam("rememberMe")){
				setcookie("AGORAP_LOG", $login, (time()+315360000), null, null, false, true);
				setcookie("AGORAP_PASS", $passwordSha1, (time()+315360000), null, null, false, true);
			}
		}
	}

	/*
	 * Selection de l'espace courant
	 */
	public static function curSpaceSelection()
	{
		//User venant d'être identifié via Post/Cookie  OU  Redir. depuis la page de connexion : Redir auto d'user déjà connecté || Espace demandé (Switch d'espace par un user - Accès à un espace par un "guest")
		if(self::$userHasConnected==true  ||  (static::moduleName=="offline" && (self::$curUser->isUser() || Req::isParam("_idSpaceAccess"))))
		{
			////	Liste des espaces de l'user.. et redirection si aucun espace dispo
			$idSpaceSelected=null;
			$spacesOfCurUser=self::$curUser->getSpaces();
			////	Aucun espace dispo (pas en page de connexion, sinon affiche une erreur avec les notifs mail de créa d'objet qui contiennent un "_idSpaceAccess"..)
			if(empty($spacesOfCurUser) && static::moduleName!="offline"){
				self::addNotif("NOTIF_noSpaceAccess");
				self::redir("?disconnect=1");
			}
			////	Espace demandé (switch d'user / accès de Guest)
			elseif(Req::isParam("_idSpaceAccess")){
				foreach($spacesOfCurUser as $objSpace){
					if($objSpace->_id==Req::getParam("_idSpaceAccess") && (self::$curUser->isUser() || empty($objSpace->password) || $objSpace->password==Req::getParam("password")))   {$idSpaceSelected=$objSpace->_id;  break;}
				}
			}
			////	Espace de connexion de l'utilisateur OU espace par defaut
			elseif(self::$curUser->isUser())
			{
				if(!empty(self::$curUser->connectionSpace)){
					foreach($spacesOfCurUser as $objSpace){
						if($objSpace->_id==self::$curUser->connectionSpace)    {$idSpaceSelected=$objSpace->_id;  break;}
					}
				}
				if(empty($idSpaceSelected)){
					$firstSpace=reset($spacesOfCurUser);
					$idSpaceSelected=$firstSpace->_id;
				}
			}
			////	Chargement de l'espace & Redirection
			if(!empty($idSpaceSelected)){
				$_SESSION["_idSpace"]=$idSpaceSelected;
				$spaceModules=self::getObj("space",$idSpaceSelected)->moduleList();
				if(Req::isParam("targetObjUrl") && self::$curUser->isUser())	{self::redir(Req::getParam("targetObjUrl"));}//Redir vers le controleur/objet demandé (notif)
				if(!empty($spaceModules))										{self::redir("?ctrl=".key($spaceModules));}//Redir vers le premier module de l'espace
				else															{self::addNotif("NOTIF_noSpaceAccess");   self::redir("?disconnect=1");}//Aucun module dans l'espace..
			}
			////	User identifié : Aucun espace dispo..
			elseif(self::$curUser->isUser()){
				self::addNotif("NOTIF_noSpaceAccess");
				self::redir("?disconnect=1");
			}
		}
		//Sortie de l'espace si aucun espace sélectionné & controleur interne sélectionné..
		elseif(empty(self::$curSpace->_id) && static::moduleName!="offline")    {self::redir("?disconnect=1");}
	}

	/*
	 * Recupère un objet avec "self::getObj()" et le "targetObjId" en GET/POST.  Exple: $_GET['targetObjId']="fileFolder-19"  (ou "fileFolder" pour un nouvel objet)
	 * Les "targetObjId" ont un controle d'accès automatique via "initCtrl()" !
	 */
	public static function getTargetObj($targetObjId=null)
	{
		//Aucun $targetObjId en paramètre et "targetObjId" spécifié par Get/Post
		if($targetObjId==null && Req::isParam("targetObjId"))	{$targetObjId=Req::getParam("targetObjId");}
		//renvoie l'objet ciblé
		if(!empty($targetObjId))
		{
			//Nouvel objet / Objet existant
			$targetObjId=explode("-",$targetObjId);
			$targetObj=(empty($targetObjId[1]))  ?  self::getObj($targetObjId[0])  :  self::getObj($targetObjId[0],$targetObjId[1]);
			//Ajoute "_idContainer" pour le controle d'accès d'un nouvel objet (cf "actionBiduleEdit()")
			if(Req::isParam("_idContainer") && empty($targetObj->_id) && empty($targetObj->_idContainer))  {$targetObj->_idContainer=Req::getParam("_idContainer");}
			//renvoie l'objet
			return $targetObj;
		}
	}

	/*
	 * Recupère les objets selectionnés et envoyés via GET/POST. Exple: $_GET['targetObjects[fileFolder]']="2-4-7"
	 */
	public static function getTargetObjects($objectType=null)
	{
		$returnObjects=array();
		if(Req::isParam("targetObjects") && is_array(Req::getParam("targetObjects")))
		{
			//On parcourt tous les objets ciblés
			foreach(Req::getParam("targetObjects") as $tmpObjectType=>$tmpObjectIds)
			{
				//Ajoute tous les types d'objets / un type en particulier
				if($objectType==null || $tmpObjectType==$objectType)
				{
					//Ajoute les objets s'ils sont accessibles
					foreach(explode("-",$tmpObjectIds) as $tmpObjectId){
						$tmpObject=self::getObj($tmpObjectType, $tmpObjectId);
						if($tmpObject->readRight())  {$returnObjects[]=$tmpObject;}
					}
				}
			}
		}
		//Retourne les objets
		return $returnObjects;
	}

	/*
	 * Récupère une préférence  (tri des résultats/type d'affichage/etc)
	 * Passé en parametre GET/POST ? Enregistre en BDD ?
	 */
	public static function prefUser($prefDbKey, $prefParamKey=null, $emptyValueEnabled=false)
	{
		//Clé identique en BDD et en GET-POST ?
		if(empty($prefParamKey))  {$prefParamKey=$prefDbKey;}
		//Préférence passé en Get/Post ?
		if(Req::isParam($prefParamKey))										{$prefParamVal=Req::getParam($prefParamKey);}
		elseif($emptyValueEnabled==true && Req::isParam("formValidate"))	{$prefParamVal="";}//Enregistre une valeur vide? (exple: checkbox non cochée dans un formulaire)
		//Enregistre si besoin la préférence  ("isset" pour aussi enregistrer les valeurs vides) 
		if(isset($prefParamVal))
		{
			//Formate la valeur
			if(is_array($prefParamVal))  {$prefParamVal=Txt::tab2txt($prefParamVal);}
			//User : enregistre en Bdd
			if(self::$curUser->isUser()){
				Db::query("DELETE FROM ap_userPreference WHERE _idUser=".self::$curUser->_id." AND keyVal=".Db::format($prefDbKey));
				Db::query("INSERT INTO ap_userPreference SET _idUser=".self::$curUser->_id.", keyVal=".Db::format($prefDbKey).", value=".Db::format($prefParamVal));
			}
			//Enregistre en session
			$_SESSION["pref"][$prefDbKey]=$prefParamVal;
		}
		//retourne la preference
		if(isset($_SESSION["pref"][$prefDbKey]))	{return $_SESSION["pref"][$prefDbKey];}
	}

	/*
	 * Ajout d'un log
	 * Action : "connexion", "add", "modif", "delete"
	 */
	public static function addLog($action, $curObj=null, $comment=null)
	{
		//S'il s'agit d'une action d'un user ou d'un invité qui ajoute un élément
		if(self::$curUser->isUser() || $action=="add")
		{
			////	Init la requête Sql
			$moduleName=Req::$curCtrl;
			$sqlObjectType=$sqlObjectId=null;
			$sqlValues="date=".Db::dateNow().", _idUser=".Db::format(self::$curUser->_id).", _idSpace=".Db::format(self::$curSpace->_id).", ip=".Db::format($_SERVER["REMOTE_ADDR"]);
			////	Element : ajoute les détails (nom, titre, chemin, etc)
			if(is_object($curObj) && $curObj->isNew()==false)
			{
				//init
				$moduleName=$curObj::moduleName;
				$sqlObjectType=$curObj::objectType;
				$sqlObjectId=$curObj->_id;
				if(!empty($comment))	{$comment.=" - ";}
				//Commentaire de base : nom / titre / description / adresse 
				if(!empty($curObj->name))			{$comment.=$curObj->name;}
				elseif(!empty($curObj->title))		{$comment.=Txt::reduce($curObj->title);}
				elseif(!empty($curObj->description)){$comment.=Txt::reduce($curObj->description);}
				elseif(!empty($curObj->adress))		{$comment.=Txt::reduce($curObj->adress);}
				//Commentaires détaillé : auteur invité / chemin de l'objet (arbo)
				if(self::$curUser->isUser()==false)							{$comment.=" - ".Txt::trad("guest")." : ".$curObj->guest;}//Auteur Invité
				if($curObj::isInArbo() && $curObj->isRootFolder()==false)	{$comment.=" - chemin : ".$curObj->containerObj()->folderPath("zip");}//Chemin de l'objet (format zip : minimaliste)
				//On ne dépasse pas les 300 carac en bdd (sinon renvoie une erreur)
				$comment=Txt::reduce($comment,290);
			}
			////	Ajoute le log
			Db::query("INSERT INTO ap_log SET action=".Db::format($action).", moduleName=".Db::format($moduleName).", objectType=".Db::format($sqlObjectType).", _idObject=".Db::format($sqlObjectId).", comment=".Db::format($comment).", ".$sqlValues);
			////	Supprime les anciens logs
			Db::query("DELETE FROM ap_log WHERE action='connexion' AND UNIX_TIMESTAMP(date) <= ".intval(time()-(7*86400)));						//logs de plus d'une semaine
			Db::query("DELETE FROM ap_log WHERE action='modif' AND UNIX_TIMESTAMP(date) <= ".intval(time()-(360*86400)));						//logs de plus d'un an
			Db::query("DELETE FROM ap_log WHERE action='delete' AND UNIX_TIMESTAMP(date) <= ".intval(time()-(self::$agora->logsTimeOut*86400)));//logs en fonction du "logsTimeOut"
		}
	}

	/*
	 * Recupere les plugins de type "Folder" d'un module
	 */
	public static function getPluginsFolders($pluginParams, $MdlObjectFolder)
	{
		$pluginsList=[];
		foreach($MdlObjectFolder::getPluginObjects($pluginParams) as $objFolder)
		{
			$objFolder->pluginModule=static::moduleName;
			$objFolder->pluginIcon="folder/folderSmall.png";
			$objFolder->pluginLabel=$objFolder->name;
			$objFolder->pluginTooltip=$objFolder->folderPath("text")."<br>".(!empty($objFolder->description)?$objFolder->description."<br>":null).$objFolder->displayAutor(true,true);
			$objFolder->pluginJsIcon="redir('".$objFolder->getUrl("container")."',true);";
			$objFolder->pluginJsLabel=$objFolder->pluginJsIcon;
			$objFolder->pluginIsFolder=true;
			$pluginsList[]=$objFolder;
		}
		return $pluginsList;
	}
}